@@ -57,18 +57,21 @@ def find_crossplot_scores(df, x, y, algo='comb', **kwargs):
     if len(tmp)<10:
         return [], {}, tmp.index
 
-    if (algo=='comb'):
-        elienv = EllipticEnvelope(
-            contamination = EliEnv_parameters['contamination'],
-            random_state = EliEnv_parameters['random_state']
-        ).fit(tmp[[x, y]])
-        preds2 = elienv.predict(tmp[[x, y]])
+    if (algo=='comb') or (algo=='EliEnv'):
+        try:
+            elienv = EllipticEnvelope(
+                contamination = EliEnv_parameters['contamination'],
+                random_state = EliEnv_parameters['random_state']
+            ).fit(tmp[[x, y]])    
+            preds2 = elienv.predict(tmp[[x, y]])
+        except:
+            preds2 = np.zeros(len(tmp))
         if algo=='EliEnv':
             preds = preds2
         original_scores = elienv.score_samples(tmp[[x, y]])
         method_scores['EliEnv'] = np.abs(standardizer(original_scores.reshape(-1,1)))
 
-    if (algo=='comb'):
+    if (algo=='comb') or (algo=='svm'):
         svm = OneClassSVM(
             kernel='rbf', 
             gamma='scale', 
@@ -80,14 +83,13 @@ def find_crossplot_scores(df, x, y, algo='comb', **kwargs):
         original_scores = svm.score_samples(tmp[[x, y]])
         method_scores['SVM'] = np.abs(standardizer(original_scores.reshape(-1,1)))
 
-    if (algo=='comb'):
+    if (algo=='comb') or (algo=='isofor'):
         ifo = IsolationForest(
             n_estimators=IsoFor_params['n_estimators'], 
             contamination=IsoFor_params['contamination'], 
             random_state=IsoFor_params['random_state']
         ).fit(tmp[[x, y]])
         preds4 = ifo.predict(tmp[[x, y]])
-
         if algo=='isofor':
             preds = preds4
         original_scores = ifo.score_samples(tmp[[x, y]])
