@@ -1,4 +1,6 @@
 import numpy as np
+
+### Flatline anomalies
 def get_constant_derivatives(df, logname, th=0.0005):
     """
     Returns indices of anomalous samples based on derivatives lower than a threshold
@@ -48,11 +50,14 @@ def get_den_flatlines(df, th=0.002):
     Returns:
         list: indices of anomalous density samples
     """
-    df['DEN_flat'] = np.gradient(df['DEN'])
-    idx = df[
-        (np.abs(df['DEN_flat'].diff())<th) &\
-        (np.abs(df['DEN_flat'].diff(-1))<th)
-    ].index
+    if len(df['DEN']):
+        df['DEN_flat'] = np.gradient(df['DEN'])
+        idx = df[
+            (np.abs(df['DEN_flat'].diff())<th) &\
+            (np.abs(df['DEN_flat'].diff(-1))<th)
+        ].index
+    else:
+        idx = []
     return idx
 
 def get_flatlines(df, logname, cols, window=5, var_window=10, n_cols=2):
@@ -107,8 +112,8 @@ def flag_flatline(df_well, y_pred=None, **kwargs):
         df_well.loc[:, col+'_gradient'] = np.abs(np.gradient(df_well[col]))
 
     def run_get_flatlines(df_well, cols, ncols, **kwargs):
-        window_size     = kwargs.get('window', 5)
-        var_window_size = kwargs.get('var_window', 10)
+        window_size     = kwargs.get('window', 10)
+        var_window_size = kwargs.get('var_window', 20)
         ac_grad_anomalies = get_flatlines(
             df=df_well, 
             logname='AC', 
@@ -133,11 +138,39 @@ def flag_flatline(df_well, y_pred=None, **kwargs):
             var_window=var_window_size,
             n_cols=ncols
         )
-        return ac_grad_anomalies, acs_grad_anomalies, den_grad_anomalies
+        return ac_grad_anomalies.tolist(), acs_grad_anomalies.tolist(), den_grad_anomalies.tolist()
 
-    #run the function for all points
-    ac_grad_anomalies, acs_grad_anomalies, den_grad_anomalies = run_get_flatlines(df_well, cols, ncols, **kwargs)
+    ac_grad_anomalies  = []
+    acs_grad_anomalies = []
+    den_grad_anomalies = []
 
+    ac_flat_grad, ac_derivatives   = get_constant_derivatives(df_well, 'AC')
+    acs_flat_grad, acs_derivatives = get_constant_derivatives(df_well, 'ACS')
+    den_flat_grad, den_derivatives = get_constant_derivatives(df_well, 'DEN')
+
+    ac_grad_anomalies.extend(ac_flat_grad)
+    acs_grad_anomalies.extend(acs_flat_grad)
+    den_grad_anomalies.extend(den_flat_grad)
+    
+    ac_flat_window, ac_window_grad   = get_constant_windows(df_well, 'AC')
+    acs_flat_window, acs_window_grad = get_constant_windows(df_well, 'ACS')
+    den_flat_window, den_window_grad = get_constant_windows(df_well, 'DEN')
+
+    ac_grad_anomalies.extend(ac_flat_window)
+    acs_grad_anomalies.extend(acs_flat_window)
+    den_grad_anomalies.extend(den_flat_window)
+
+    gen_grad_anomalies = []
+    gen_grad_anomalies.extend(ac_grad_anomalies)
+    gen_grad_anomalies.extend(acs_grad_anomalies)
+    gen_grad_anomalies.extend(den_grad_anomalies)
+
+    #run the function on points where there was constant window or gradient flags
+    ac_grad_anomalies_main, acs_grad_anomalies_main, den_grad_anomalies_main = run_get_flatlines(
+        df_well.loc[gen_grad_anomalies], cols, ncols, **kwargs)
+    
+    den_grad_anomalies_main.extend(get_den_flatlines(df_well.loc[gen_grad_anomalies]))
+    
     #pick the start and end of well
     start_end_length = 50
     start_end_window_size = 2
@@ -147,7 +180,6 @@ def flag_flatline(df_well, y_pred=None, **kwargs):
     end_condition = (df_well.index>=max_idx-start_end_length) & (df_well.index<=max_idx)
 
     #run the function for end points only with smaller window sizes
-    
     df_well_ends = df_well[start_condition | end_condition]
     ac_grad_anomalies_ends, acs_grad_anomalies_ends, den_grad_anomalies_ends = run_get_flatlines(
         df_well_ends,
@@ -156,44 +188,30 @@ def flag_flatline(df_well, y_pred=None, **kwargs):
         **{'window': start_end_window_size, 'var_window': start_end_var_window_size}
         )
 
-    ac_grad_anomalies  = ac_grad_anomalies.append(ac_grad_anomalies_ends)
-    acs_grad_anomalies = acs_grad_anomalies.append(acs_grad_anomalies_ends)
-    den_grad_anomalies = den_grad_anomalies.append(den_grad_anomalies_ends)
-
-
-    ac_flat_grad, ac_derivatives   = get_constant_derivatives(df_well, 'AC')
-    acs_flat_grad, acs_derivatives = get_constant_derivatives(df_well, 'ACS')
-    den_flat_grad, den_derivatives = get_constant_derivatives(df_well, 'DEN')
-
-    ac_grad_anomalies.append(ac_flat_grad)
-    acs_grad_anomalies.append(acs_flat_grad)
-    den_grad_anomalies.append(den_flat_grad)
-    
-    ac_flat_window, ac_window_grad   = get_constant_windows(df_well, 'AC')
-    acs_flat_window, acs_window_grad = get_constant_windows(df_well, 'ACS')
-    den_flat_window, den_window_grad = get_constant_windows(df_well, 'DEN')
-
-    ac_grad_anomalies  = ac_grad_anomalies.append(ac_flat_window)
-    acs_grad_anomalies = acs_grad_anomalies.append(acs_flat_window)
-    den_grad_anomalies = den_grad_anomalies.append(den_flat_window)
-
-    den_grad_anomalies = den_grad_anomalies.append(get_den_flatlines(df_well))
+    ac_grad_anomalies_main.extend(ac_grad_anomalies_ends)
+    acs_grad_anomalies_main.extend(acs_grad_anomalies_ends)
+    den_grad_anomalies_main.extend(den_grad_anomalies_ends)
 
     if y_pred is None:
         y_pred = df_well.copy()
 
-    y_pred['score_flat_deriv_ac']  = ac_derivatives.fillna(0)
-    y_pred['score_flat_deriv_acs'] = acs_derivatives.fillna(0)
-    y_pred['score_flat_deriv_den'] = den_derivatives.fillna(0)
-
-    y_pred['score_flat_window_ac']  = ac_window_grad.fillna(0)
-    y_pred['score_flat_window_acs'] = acs_window_grad.fillna(0)
-    y_pred['score_flat_window_den'] = den_window_grad.fillna(0)
-
     y_pred.loc[:, ['flag_flatline_gen', 'flag_flatline_ac', 'flag_flatline_acs', 'flag_flatline_den']] =\
         0, 0, 0, 0
-    y_pred.loc[ac_grad_anomalies, ['flag_flatline_gen', 'flag_flatline_ac']]   = 1
-    y_pred.loc[acs_grad_anomalies, ['flag_flatline_gen', 'flag_flatline_acs']] = 1
-    y_pred.loc[den_grad_anomalies, ['flag_flatline_gen', 'flag_flatline_den']] = 1
+
+    #DEBUG   
+    flag_columns = ['flag_den_only']
+    lognames = ['ac', 'acs', 'den']
+    for logname in lognames:
+        flag_columns.extend([
+            'flag_main_{}'.format(logname), 
+            'flag_ends_{}'.format(logname), 
+            'flag_derivs_{}'.format(logname), 
+            'flag_windows_{}'.format(logname)
+            ])
+    y_pred.loc[:, flag_columns] = 0
+
+    y_pred.loc[ac_grad_anomalies_main, ['flag_flatline_gen', 'flag_flatline_ac']]   = 1
+    y_pred.loc[acs_grad_anomalies_main, ['flag_flatline_gen', 'flag_flatline_acs']] = 1
+    y_pred.loc[den_grad_anomalies_main, ['flag_flatline_gen', 'flag_flatline_den']] = 1
 
     return y_pred
\ No newline at end of file
